==> ui-18.3.2.tgz: package/components/popover/Popover.js <==
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { isString, isUndefined } from 'lodash';
import React, { useEffect, useImperativeHandle, useRef, useState } from 'react';
import ReactDOM from 'react-dom';
import { useEvent, useEventCallback, useId, useLockScroll, useRefExtra } from '@react-devui/hooks';
import { getClassName, getPopupPosition } from '@react-devui/utils';
import { useMaxIndex, useDValue } from '../../hooks';
import { registerComponentMate, handleModalKeyDown, cloneHTMLElement, checkNoExpandedEl } from '../../utils';
import { ESC_CLOSABLE_DATA } from '../../utils/checkNoExpandedEl';
import { DPopup } from '../_popup';
import { DTransition } from '../_transition';
import { useComponentConfig, usePrefixConfig } from '../root';
import { DPopoverFooter } from './PopoverFooter';
import { DPopoverHeader } from './PopoverHeader';
const TTANSITION_DURING = { enter: 86, leave: 100 };
const { COMPONENT_NAME } = registerComponentMate({ COMPONENT_NAME: 'DPopover' });
function Popover(props, ref) {
    var _a;
    const _b = useComponentConfig(COMPONENT_NAME, props), { children, dVisible, dTrigger = 'hover', dContainer, dPlacement = 'top', dEscClosable = true, dDestroyAfterClose = false, dArrow = true, dModal = false, dDistance = 10, dInWindow = false, dMouseEnterDelay = 150, dMouseLeaveDelay = 200, dZIndex, dHeader, dContent, dFooter, onVisibleChange, afterVisibleChange } = _b, restProps = __rest(_b, ["children", "dVisible", "dTrigger", "dContainer", "dPlacement", "dEscClosable", "dDestroyAfterClose", "dArrow", "dModal", "dDistance", "dInWindow", "dMouseEnterDelay", "dMouseLeaveDelay", "dZIndex", "dHeader", "dContent", "dFooter", "onVisibleChange", "afterVisibleChange"]);
    //#region Context
    const dPrefix = usePrefixConfig();
    //#endregion
    //#region Ref
    const windowRef = useRefExtra(() => window);
    const triggerRef = useRefExtra(() => document.getElementById(triggerId));
    const popoverRef = useRef(null);
    const popupRef = useRef(null);
    const containerRef = useRefExtra(isUndefined(dContainer)
        ? () => {
            let el = document.getElementById(`${dPrefix}popover-root`);
            if (!el) {
                el = document.createElement('div');
                el.id = `${dPrefix}popover-root`;
                document.body.appendChild(el);
            }
            return el;
        }
        : dContainer === false
            ? () => {
                var _a, _b;
                return (_b = (_a = triggerRef.current) === null || _a === void 0 ? void 0 : _a.parentElement) !== null && _b !== void 0 ? _b : null;
            }
            : dContainer, true);
    //#endregion
    const dataRef = useRef({
        prevActiveEl: null,
    });
    const uniqueId = useId();
    const triggerId = (_a = children.props.id) !== null && _a !== void 0 ? _a : `${dPrefix}popover-trigger-${uniqueId}`;
    const titleId = `${dPrefix}popover-title-${uniqueId}`;
    const bodyId = `${dPrefix}popover-content-${uniqueId}`;
    const [visible, changeVisible] = useDValue(false, dVisible, onVisibleChange);
    const maxZIndex = useMaxIndex(visible);
    const zIndex = (() => {
        if (isUndefined(dZIndex)) {
            if (isUndefined(dContainer)) {
                return maxZIndex;
            }
            else {
                return `var(--${dPrefix}zindex-absolute)`;
            }
        }
        else {
            return dZIndex;
        }
    })();
    const [popupPositionStyle, setPopupPositionStyle] = useState({
        top: '-200vh',
        left: '-200vw',
    });
    const [placement, setPlacement] = useState(dPlacement);
    const [transformOrigin, setTransformOrigin] = useState();
    const updatePosition = useEventCallback(() => {
        if (visible && popupRef.current && triggerRef.current) {
            let currentPlacement = dPlacement;
            let space = [0, 0, 0, 0];
            if (!isUndefined(dContainer) && containerRef.current) {
                const containerRect = containerRef.current.getBoundingClientRect();
                space = [
                    containerRect.top,
                    window.innerWidth - containerRect.left - containerRect.width,
                    window.innerHeight - containerRect.top - containerRect.height,
                    containerRect.left,
                ];
            }
            const position = getPopupPosition(popupRef.current, triggerRef.current, {
                placement: dPlacement,
                offset: dDistance,
                inWindow: dInWindow,
            }, space);
            if (position) {
                currentPlacement = position.placement;
                setPlacement(position.placement);
                setPopupPositionStyle({
                    top: position.top,
                    left: position.left,
                });
            }
            else {
                const position = getPopupPosition(popupRef.current, triggerRef.current, {
                    placement,
                    offset: dDistance,
                    inWindow: dInWindow,
                });
                setPopupPositionStyle({
                    top: position.top,
                    left: position.left,
                });
            }
            let transformOrigin = 'center bottom';
            switch (currentPlacement) {
                case 'top':
                    transformOrigin = 'center bottom';
                    break;
                case 'top-left':
                    transformOrigin = '20px bottom';
                    break;
                case 'top-right':
                    transformOrigin = 'calc(100% - 20px) bottom';
                    break;
                case 'right':
                    transformOrigin = 'left center';
                    break;
                case 'right-top':
                    transformOrigin = 'left 12px';
                    break;
                case 'right-bottom':
                    transformOrigin = 'left calc(100% - 12px)';
                    break;
                case 'bottom':
                    transformOrigin = 'center top';
                    break;
                case 'bottom-left':
                    transformOrigin = '20px top';
                    break;
                case 'bottom-right':
                    transformOrigin = 'calc(100% - 20px) top';
                    break;
                case 'left':
                    transformOrigin = 'right center';
                    break;
                case 'left-top':
                    transformOrigin = 'right 12px';
                    break;
                case 'left-bottom':
                    transformOrigin = 'right calc(100% - 12px)';
                    break;
                default:
                    break;
            }
            setTransformOrigin(transformOrigin);
        }
    });
    useLockScroll(dModal && visible);
    useEvent(windowRef, 'keydown', (e) => {
        if (e.code === 'Escape' && popoverRef.current && checkNoExpandedEl(popoverRef.current)) {
            changeVisible(false);
        }
    }, {}, !dEscClosable || !visible);
    useEffect(() => {
        if (dModal) {
            if (visible) {
                dataRef.current.prevActiveEl = document.activeElement;
                if (popoverRef.current) {
                    popoverRef.current.focus({ preventScroll: true });
                }
            }
            else if (dataRef.current.prevActiveEl) {
                dataRef.current.prevActiveEl.focus({ preventScroll: true });
            }
        }
    }, [dModal, visible]);
    const headerNode = (() => {
        if (dHeader) {
            const node = isString(dHeader) ? _jsx(DPopoverHeader, { children: dHeader }) : dHeader;
            return React.cloneElement(node, {
                __id: titleId,
                __onClose: () => {
                    changeVisible(false);
                },
            });
        }
    })();
    useImperativeHandle(ref, () => ({
        updatePosition,
    }), [updatePosition]);
    return (_jsx(DPopup, Object.assign({ dVisible: visible, dTrigger: dTrigger, dMouseEnterDelay: dMouseEnterDelay, dMouseLeaveDelay: dMouseLeaveDelay, dUpdatePosition: {
            fn: updatePosition,
            triggerRef,
            popupRef,
            extraScrollRefs: [isUndefined(dContainer) ? undefined : containerRef],
        }, onVisibleChange: changeVisible }, { children: ({ renderTrigger, renderPopup }) => (_jsxs(_Fragment, { children: [renderTrigger(cloneHTMLElement(children, {
                    id: triggerId,
                    [ESC_CLOSABLE_DATA]: dEscClosable && visible,
                })), containerRef.current &&
                    ReactDOM.createPortal(_jsx(DTransition, Object.assign({ dIn: visible, dDuring: TTANSITION_DURING, dDestroyWhenLeaved: dDestroyAfterClose, onEnter: updatePosition, afterEnter: () => {
                            afterVisibleChange === null || afterVisibleChange === void 0 ? void 0 : afterVisibleChange(true);
                        }, afterLeave: () => {
                            afterVisibleChange === null || afterVisibleChange === void 0 ? void 0 : afterVisibleChange(false);
                        } }, { children: (state) => {
                            var _a;
                            let transitionStyle = {};
                            switch (state) {
                                case 'enter':
                                    transitionStyle = { transform: 'scale(0.3)', opacity: 0 };
                                    break;
                                case 'entering':
                                    transitionStyle = {
                                        transition: ['transform', 'opacity'].map((attr) => `${attr} ${TTANSITION_DURING.enter}ms ease-out`).join(', '),
                                        transformOrigin,
                                    };
                                    break;
                                case 'leaving':
                                    transitionStyle = {
                                        transform: 'scale(0.3)',
                                        opacity: 0,
                                        transition: ['transform', 'opacity'].map((attr) => `${attr} ${TTANSITION_DURING.leave}ms ease-in`).join(', '),
                                        transformOrigin,
                                    };
                                    break;
                                default:
                                    break;
                            }
                            return (_jsxs("div", Object.assign({}, restProps, { ref: popoverRef, className: getClassName(restProps.className, `${dPrefix}popover`, `${dPrefix}popover--` + placement), style: Object.assign(Object.assign({}, restProps.style), { display: state === 'leaved' ? 'none' : undefined, zIndex }), tabIndex: -1, role: (_a = restProps.role) !== null && _a !== void 0 ? _a : 'dialog', "aria-modal": dModal, "aria-labelledby": headerNode ? titleId : undefined, "aria-describedby": bodyId, onKeyDown: (e) => {
                                    var _a;
                                    (_a = restProps.onKeyDown) === null || _a === void 0 ? void 0 : _a.call(restProps, e);
                                    if (dEscClosable && e.code === 'Escape') {
                                        changeVisible(false);
                                    }
                                    if (dModal) {
                                        handleModalKeyDown(e);
                                    }
                                } }, { children: [dModal && (_jsx("div", { className: `${dPrefix}popover__mask`, onClick: () => {
                                            changeVisible(false);
                                        } })), renderPopup(_jsxs("div", Object.assign({ ref: popupRef, className: `${dPrefix}popover__content`, style: Object.assign(Object.assign({}, popupPositionStyle), transitionStyle) }, { children: [dArrow && _jsx("div", { className: `${dPrefix}popover__arrow` }), headerNode, _jsx("div", Object.assign({ id: bodyId, className: `${dPrefix}popover__body` }, { children: dContent })), dFooter &&
                                                React.cloneElement(dFooter, {
                                                    __onClose: () => {
                                                        changeVisible(false);
                                                    },
                                                })] })))] })));
                        } })), containerRef.current)] })) })));
}
export const DPopover = React.forwardRef(Popover);
DPopover.Header = DPopoverHeader;
DPopover.Footer = DPopoverFooter;

==> ui-18.5.7.tgz: package/components/popover/Popover.js <==
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { isString, isUndefined } from 'lodash';
import React, { useImperativeHandle, useRef, useState } from 'react';
import ReactDOM from 'react-dom';
import { useEvent, useEventCallback, useId, useLockScroll, useRefExtra } from '@react-devui/hooks';
import { getClassName, getPopupPosition } from '@react-devui/utils';
import { useMaxIndex, useDValue } from '../../hooks';
import { registerComponentMate, handleModalKeyDown, cloneHTMLElement, checkNoExpandedEl } from '../../utils';
import { ESC_CLOSABLE_DATA } from '../../utils/checkNoExpandedEl';
import { DPopup } from '../_popup';
import { DTransition } from '../_transition';
import { useComponentConfig, usePrefixConfig } from '../root';
import { DPopoverFooter } from './PopoverFooter';
import { DPopoverHeader } from './PopoverHeader';
const TTANSITION_DURING = { enter: 86, leave: 100 };
const { COMPONENT_NAME } = registerComponentMate({ COMPONENT_NAME: 'DPopover' });
function Popover(props, ref) {
    var _a;
    const _b = useComponentConfig(COMPONENT_NAME, props), { children, dVisible, dTrigger = 'hover', dContainer, dPlacement = 'top', dEscClosable = true, dDestroyAfterClose = false, dArrow = true, dModal = false, dDistance = 10, dInWindow = false, dMouseEnterDelay = 150, dMouseLeaveDelay = 200, dZIndex, dHeader, dContent, dFooter, onVisibleChange, afterVisibleChange } = _b, restProps = __rest(_b, ["children", "dVisible", "dTrigger", "dContainer", "dPlacement", "dEscClosable", "dDestroyAfterClose", "dArrow", "dModal", "dDistance", "dInWindow", "dMouseEnterDelay", "dMouseLeaveDelay", "dZIndex", "dHeader", "dContent", "dFooter", "onVisibleChange", "afterVisibleChange"]);
    //#region Context
    const dPrefix = usePrefixConfig();
    //#endregion
    //#region Ref
    const windowRef = useRefExtra(() => window);
    const triggerRef = useRefExtra(() => document.getElementById(triggerId));
    const popoverRef = useRef(null);
    const popupRef = useRef(null);
    const containerRef = useRefExtra(isUndefined(dContainer)
        ? () => {
            let el = document.getElementById(`${dPrefix}popover-root`);
            if (!el) {
                el = document.createElement('div');
                el.id = `${dPrefix}popover-root`;
                document.body.appendChild(el);
            }
            return el;
        }
        : dContainer === false
            ? () => {
                var _a, _b;
                return (_b = (_a = triggerRef.current) === null || _a === void 0 ? void 0 : _a.parentElement) !== null && _b !== void 0 ? _b : null;
            }
            : dContainer, true);
    //#endregion
    const dataRef = useRef({
        prevActiveEl: null,
    });
    const uniqueId = useId();
    const triggerId = (_a = children.props.id) !== null && _a !== void 0 ? _a : `${dPrefix}popover-trigger-${uniqueId}`;
    const titleId = `${dPrefix}popover-title-${uniqueId}`;
    const bodyId = `${dPrefix}popover-content-${uniqueId}`;
    const [visible, changeVisible] = useDValue(false, dVisible, onVisibleChange);
    const maxZIndex = useMaxIndex(visible);
    const zIndex = (() => {
        if (isUndefined(dZIndex)) {
            if (isUndefined(dContainer)) {
                return maxZIndex;
            }
            else {
                return `var(--${dPrefix}zindex-absolute)`;
            }
        }
        else {
            return dZIndex;
        }
    })();
    const [popupPositionStyle, setPopupPositionStyle] = useState({
        top: '-200vh',
        left: '-200vw',
    });
    const [placement, setPlacement] = useState(dPlacement);
    const [transformOrigin, setTransformOrigin] = useState();
    const updatePosition = useEventCallback(() => {
        if (visible && popupRef.current && triggerRef.current) {
            let currentPlacement = dPlacement;
            let space = [0, 0, 0, 0];
            if (!isUndefined(dContainer) && containerRef.current) {
                const containerRect = containerRef.current.getBoundingClientRect();
                space = [
                    containerRect.top,
                    window.innerWidth - containerRect.left - containerRect.width,
                    window.innerHeight - containerRect.top - containerRect.height,
                    containerRect.left,
                ];
            }
            const position = getPopupPosition(popupRef.current, triggerRef.current, {
                placement: dPlacement,
                offset: dDistance,
                inWindow: dInWindow,
            }, space);
            if (position) {
                currentPlacement = position.placement;
                setPlacement(position.placement);
                setPopupPositionStyle({
                    top: position.top,
                    left: position.left,
                });
            }
            else {
                const position = getPopupPosition(popupRef.current, triggerRef.current, {
                    placement,
                    offset: dDistance,
                    inWindow: dInWindow,
                });
                setPopupPositionStyle({
                    top: position.top,
                    left: position.left,
                });
            }
            let transformOrigin = 'center bottom';
            switch (currentPlacement) {
                case 'top':
                    transformOrigin = 'center bottom';
                    break;
                case 'top-left':
                    transformOrigin = '20px bottom';
                    break;
                case 'top-right':
                    transformOrigin = 'calc(100% - 20px) bottom';
                    break;
                case 'right':
                    transformOrigin = 'left center';
                    break;
                case 'right-top':
                    transformOrigin = 'left 12px';
                    break;
                case 'right-bottom':
                    transformOrigin = 'left calc(100% - 12px)';
                    break;
                case 'bottom':
                    transformOrigin = 'center top';
                    break;
                case 'bottom-left':
                    transformOrigin = '20px top';
                    break;
                case 'bottom-right':
                    transformOrigin = 'calc(100% - 20px) top';
                    break;
                case 'left':
                    transformOrigin = 'right center';
                    break;
                case 'left-top':
                    transformOrigin = 'right 12px';
                    break;
                case 'left-bottom':
                    transformOrigin = 'right calc(100% - 12px)';
                    break;
                default:
                    break;
            }
            setTransformOrigin(transformOrigin);
        }
    });
    useLockScroll(dModal && visible);
    useEvent(windowRef, 'keydown', (e) => {
        if (e.code === 'Escape' && popoverRef.current && checkNoExpandedEl(popoverRef.current)) {
            changeVisible(false);
        }
    }, {}, !dEscClosable || !visible);
    const headerNode = (() => {
        if (dHeader) {
            const node = isString(dHeader) ? _jsx(DPopoverHeader, { children: dHeader }) : dHeader;
            return React.cloneElement(node, {
                __id: titleId,
                __onClose: () => {
                    changeVisible(false);
                },
            });
        }
    })();
    useImperativeHandle(ref, () => ({
        updatePosition,
    }), [updatePosition]);
    return (_jsx(DPopup, Object.assign({ dVisible: visible, dTrigger: dTrigger, dMouseEnterDelay: dMouseEnterDelay, dMouseLeaveDelay: dMouseLeaveDelay, dUpdatePosition: {
            fn: updatePosition,
            triggerRef,
            popupRef,
            extraScrollRefs: [isUndefined(dContainer) ? undefined : containerRef],
        }, onVisibleChange: changeVisible }, { children: ({ renderTrigger, renderPopup }) => (_jsxs(_Fragment, { children: [renderTrigger(cloneHTMLElement(children, {
                    id: triggerId,
                    [ESC_CLOSABLE_DATA]: dEscClosable && visible,
                })), containerRef.current &&
                    ReactDOM.createPortal(_jsx(DTransition, Object.assign({ dIn: visible, dDuring: TTANSITION_DURING, dDestroyWhenLeaved: dDestroyAfterClose, onEnter: updatePosition, afterEnter: () => {
                            afterVisibleChange === null || afterVisibleChange === void 0 ? void 0 : afterVisibleChange(true);
                            if (dModal) {
                                dataRef.current.prevActiveEl = document.activeElement;
                                if (popoverRef.current) {
                                    popoverRef.current.focus({ preventScroll: true });
                                }
                            }
                        }, afterLeave: () => {
                            afterVisibleChange === null || afterVisibleChange === void 0 ? void 0 : afterVisibleChange(false);
                            if (dModal && dataRef.current.prevActiveEl) {
                                dataRef.current.prevActiveEl.focus({ preventScroll: true });
                            }
                        } }, { children: (state) => {
                            var _a;
                            let transitionStyle = {};
                            switch (state) {
                                case 'enter':
                                    transitionStyle = { transform: 'scale(0.3)', opacity: 0 };
                                    break;
                                case 'entering':
                                    transitionStyle = {
                                        transition: ['transform', 'opacity'].map((attr) => `${attr} ${TTANSITION_DURING.enter}ms ease-out`).join(', '),
                                        transformOrigin,
                                    };
                                    break;
                                case 'leaving':
                                    transitionStyle = {
                                        transform: 'scale(0.3)',
                                        opacity: 0,
                                        transition: ['transform', 'opacity'].map((attr) => `${attr} ${TTANSITION_DURING.leave}ms ease-in`).join(', '),
                                        transformOrigin,
                                    };
                                    break;
                                default:
                                    break;
                            }
                            return (_jsxs("div", Object.assign({}, restProps, { ref: popoverRef, className: getClassName(restProps.className, `${dPrefix}popover`, `${dPrefix}popover--` + placement), style: Object.assign(Object.assign({}, restProps.style), { display: state === 'leaved' ? 'none' : undefined, zIndex }), tabIndex: -1, role: (_a = restProps.role) !== null && _a !== void 0 ? _a : 'dialog', "aria-modal": dModal, "aria-labelledby": headerNode ? titleId : undefined, "aria-describedby": bodyId, onKeyDown: (e) => {
                                    var _a;
                                    (_a = restProps.onKeyDown) === null || _a === void 0 ? void 0 : _a.call(restProps, e);
                                    if (dEscClosable && e.code === 'Escape') {
                                        changeVisible(false);
                                    }
                                    if (dModal) {
                                        handleModalKeyDown(e);
                                    }
                                } }, { children: [dModal && (_jsx("div", { className: `${dPrefix}popover__mask`, onClick: () => {
                                            changeVisible(false);
                                        } })), renderPopup(_jsxs("div", Object.assign({ ref: popupRef, className: `${dPrefix}popover__content`, style: Object.assign(Object.assign({}, popupPositionStyle), transitionStyle) }, { children: [dArrow && _jsx("div", { className: `${dPrefix}popover__arrow` }), headerNode, _jsx("div", Object.assign({ id: bodyId, className: `${dPrefix}popover__body` }, { children: dContent })), dFooter &&
                                                React.cloneElement(dFooter, {
                                                    __onClose: () => {
                                                        changeVisible(false);
                                                    },
                                                })] })))] })));
                        } })), containerRef.current)] })) })));
}
export const DPopover = React.forwardRef(Popover);
DPopover.Header = DPopoverHeader;
DPopover.Footer = DPopoverFooter;
